package implementation;

import java.util.ArrayList;
import java.util.Random;

public class Example {

    private static boolean[][][] matchingCache;
    private static final int dimX = 3;
    private static final int dimY = 4;
    private static final int dimZ = 3;

    public static void main(String[] args) {
        generateExample(dimX, dimY, dimZ);
    }

    public static void generateExample(int dimX, int dimY, int dimZ) {
        int grid[][][] = new int[dimX][dimY][dimZ];

        matchingCache = calculateMatchingCache();

        ArrayList<Integer> stateNumbers = new ArrayList<>();

        // Get all different stateNumbers
        for (int i = 0; i < 5; i++) {
            for (int i2 = 0; i2 < 5; i2++) {
                for (int i3 = 0; i3 < 5; i3++) {
                    for (int i4 = 0; i4 < 5; i4++) {
                        for (int i5 = 0; i5 < 5; i5++) {
                            for (int i6 = 0; i6 < 5; i6++) {
                                int stateNumber = i * 100000 + i2 * 10000 + i3 * 1000 + i4 * 100 + i5 * 10 + i6;

                                if (!stateNumbers.contains(stateNumber)) {
                                    stateNumbers.add(stateNumber);
                                }
                            }
                        }
                    }
                }
            }
        }

        Random r = new Random();

        // Generate an Example
        for (int x = 0; x < dimX; x++) {
            for (int y = 0; y < dimY; y++) {
                for (int z = 0; z < dimZ; z++) {
                    ArrayList<Integer> possibleNumbers = getPossibleStateNumbers(x, y, z, grid, stateNumbers);
                    int index = r.nextInt(possibleNumbers.size());
                    grid[x][y][z] = possibleNumbers.get(index);
                }
            }
        }

        // Ausgabe
        ArrayList<String> teile = new ArrayList<>();
        int i = 1;
        for (int x = 0; x < dimX; x++) {
            for (int y = 0; y < dimY; y++) {
                for (int z = 0; z < dimZ; z++) {
                    teile.add("Teil " + i++ + ": " + Quader.getStateString(grid[x][y][z]));
                }
            }
        }

        // Collections.shuffle(teile);

        System.out.printf("Dimension: %d,%d,%d\n", dimX, dimY, dimZ);
        for (String line : teile) {
            System.out.println(line);
        }
    }

    private static boolean[][][] calculateMatchingCache() {
        // Direction faceOfObject faceOfSatellite
        boolean[][][] matches = new boolean[2][5][6];

        for (int i = 0; i < 2; ++i) {
            for (int o = 0; o < 5; ++o) {
                for (int s = 0; s < 6; ++s) {
                    if (s == 5) {
                        matches[i][o][s] = o == 0;
                    } else {
                        if (i == 0) {
                            matches[i][o][s] = fits(false, o, s);
                        } else {
                            matches[i][o][s] = fits(true, o, s);
                        }
                    }
                }
            }
        }

        return matches;
    }

    private static boolean fits(boolean top, int objectFace, int satelliteFace) {
        if (top) {
            if (objectFace != 0 && satelliteFace != 0 && objectFace + satelliteFace == 5) {
                return true;
            }else if(objectFace != 0 && satelliteFace == 0){
                return true;
            }
        } else {
            int test = objectFace + satelliteFace;
            if (objectFace != 0 && satelliteFace != 0 && (test == 3 || test == 7)) {
                return true;
            }else if (objectFace != 0 && satelliteFace == 0){
                return true;
            }
        }

        return false;
    }

    private static ArrayList<Integer> getPossibleStateNumbers(int x, int y, int z, int[][][] grid,
            ArrayList<Integer> allStateNumbers) {
        ArrayList<Integer> result = new ArrayList<>();

        for (int i = 0; i < allStateNumbers.size(); ++i) {
            int stateNumber = allStateNumbers.get(i);
            if (fitsInPlace(stateNumber, x, y, z, grid)) {
                result.add(stateNumber);
            }
        }

        return result;
    }

    private static boolean fitsInPlace(int stateNumber, int x, int y, int z, int[][][] grid) {
        boolean fits = true;

        int needZeros = 0;
        int countZeros = 0;
        if (x == dimX - 1) {
            needZeros++;
        }
        if(x == 0){
            needZeros++;
        }
        if (y == dimY - 1) {
            needZeros++;
        }
        if(y == 0){
            needZeros++;
        }
        if (z == dimZ - 1) {
            needZeros++;
        }
        if(z == 0){
            needZeros++;
        }

        if (stateNumber == 0) {
            return false;
        }

        // TOP
        int faceNumber = stateNumber / 100000;
        if (faceNumber == 0) {
            countZeros++;
        }

        if (z == dimZ - 1) {
            fits &= matchingCache[1][faceNumber][5];
        } else {
            fits &= matchingCache[1][faceNumber][grid[x][y][z + 1] % 10];
        }

        // BOTTOM
        faceNumber = stateNumber % 10;
        if (faceNumber == 0) {
            countZeros++;
        }

        if (z == 0) {
            fits &= matchingCache[1][faceNumber][5];
        } else {
            fits &= matchingCache[1][faceNumber][grid[x][y][z - 1] / 100000];
        }

        // FRONT
        faceNumber = stateNumber / 1000 % 10;
        if (faceNumber == 0) {
            countZeros++;
        }

        if (x == dimX - 1) {
            fits &= matchingCache[0][faceNumber][5];
        } else {
            fits &= matchingCache[0][faceNumber][grid[x + 1][y][z] / 10 % 10];
        }

        // Back
        faceNumber = stateNumber / 10 % 10;
        if (faceNumber == 0) {
            countZeros++;
        }

        if (x == 0) {
            fits &= matchingCache[0][faceNumber][5];
        } else {
            fits &= matchingCache[0][faceNumber][grid[x - 1][y][z] / 1000 % 10];
        }

        // RIGHT
        faceNumber = stateNumber / 100 % 10;
        if (faceNumber == 0) {
            countZeros++;
        }

        if (y == dimY - 1) {
            fits &= matchingCache[0][faceNumber][5];
        } else {
            fits &= matchingCache[0][faceNumber][grid[x][y + 1][z] / 10000 % 10];
        }

        // LEFT
        faceNumber = stateNumber / 10000 % 10;
        if (faceNumber == 0) {
            countZeros++;
        }
        if (y == 0) {
            fits &= matchingCache[0][faceNumber][5];
        } else {
            fits &= matchingCache[0][faceNumber][grid[x][y - 1][z] / 100 % 10];
        }
        return fits && countZeros == needZeros;
    }
}
